package com.redshape.utils.system.console;

import com.redshape.utils.system.scripts.IScriptExecutor;
import com.redshape.utils.system.scripts.IScriptListExecutor;
import com.redshape.utils.system.scripts.bash.BashScriptExecutor;
import com.redshape.utils.system.scripts.bash.BashScriptListExecutor;

import java.io.*;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/**
 * @author nikelin
 */
public class Console implements IConsole {
	private Map<Object, Collection<IScriptExecutor>> executors = new HashMap<Object, Collection<IScriptExecutor>>();

	/**
	 * For testability purpouses
	 * @param command
	 * @return
	 */
	protected IScriptExecutor createExecutorObject( String command ) {
		return new BashScriptExecutor(command);
	}

    @Override
    public boolean isDirectory(String path) throws IOException {
        return this.<File>provideFile(path, false).isDirectory();
    }

    @Override
    public void mkdir(String path) throws IOException {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
	public void deleteFile(String path) throws IOException {
		File file = this.<File>provideFile(path, false);
		if ( !file.exists() ) {
			return;
		}

		file.delete();
	}

	@Override
	public boolean checkExists(String path) throws IOException {
		return this.<File>provideFile(path, false).exists();
	}

	/**
	 * For testability purpouses
	 * @param <T>
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected <T extends IScriptExecutor> IScriptListExecutor<T> createListExecutorObject() {
		return (IScriptListExecutor<T>) new BashScriptListExecutor();
	}

	@Override
	public IScriptExecutor createExecutor(String command) {
		return this.createExecutor(this, command);
	}

	@Override
	public <T extends IScriptExecutor> IScriptListExecutor<T> createListExecutor() {
		return this.createListExecutor(this);
	}

	@Override
	public void stopScripts(Object context) {
		Collection<IScriptExecutor> executors = this.executors.get(context);
		if ( executors == null ) {
			return;
		}

		for ( IScriptExecutor executor : executors ) {
			executor.kill();
		}
	}

	@Override
	public String readFile(String path) throws IOException {
		StringBuilder builder = new StringBuilder();
		BufferedReader bufferedStream = new BufferedReader(
			new InputStreamReader( this.openReadStream(path) )
		);

		String tmp;
		while ( null != ( tmp = bufferedStream.readLine() ) ) {
			builder.append(tmp).append("\n");
		}

		return builder.toString();
	}

	protected <T> T provideFile( String path, boolean createIfNotExists ) throws IOException {
		File file = new File(path);
		if ( createIfNotExists ) {
			file.createNewFile();
		}

		return (T) file;
	}

	@Override
	public InputStream openReadStream(String path) throws IOException {
		return new FileInputStream( this.<File>provideFile(path, false) );
	}

	@Override
	public OutputStream openWriteStream(String path) throws IOException {
		return new FileOutputStream( this.<File>provideFile(path, true) );
	}

	@Override
	public IScriptExecutor createExecutor(Object context, String command) {
		IScriptExecutor executor = this.createExecutorObject(command);

		this.registerExecutor( context, executor );

		return executor;
	}

	@Override
	public <T extends IScriptExecutor> IScriptListExecutor<T>
						createListExecutor( Object context) {
		IScriptListExecutor<T> executor = this.createListExecutorObject();

		this.registerExecutor( context, executor );

		return executor;
	}

	protected void registerExecutor( Object context, IScriptExecutor executor ) {
		if ( this.executors.get(context) == null ) {
			this.executors.put( context, new HashSet<IScriptExecutor>() );
		}

		this.executors.get(context).add( executor );
	}

}